// mtguru1.cpp : Defines the entry point for the DLL application.
//
#define WIN32_LEAN_AND_MEAN  // Exclude rarely-used stuff from Windows headers
#include "stdafx.h"

#define MT4_EXPFUNC __declspec(dllexport)

#define msg(m) MessageBox(NULL,m,"",NULL)


//+------------------------------------------------------------------+
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <float.h>
#include "svm.h"
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))

int read_problem(const char *filename);
void do_cross_validation();
struct svm_parameter param;		// set by parse_command_line
struct svm_problem prob;		// set by read_problem
struct svm_model *model;
struct svm_node *x_space;
int nr_fold;

char* line;
int max_line_len = 1024;
struct svm_node *x;
int max_nr_attr = 64;
//struct svm_model* model;

//Scale variables
//char *line;
//int max_line_len = 1024;

double lower=-1.0,upper=1.0,y_lower,y_upper;
int y_scaling = 0;
double *feature_max;
double *feature_min;
double y_max = -DBL_MAX;
double y_min = DBL_MAX;
int max_index;

#define max(x,y) ((x>y)?x:y)
#define min(x,y) ((x<y)?x:y)

void output_target(double value,FILE *fps);
void output(int index, double value,FILE *fps);
char* readline(FILE *input);



//+------------------------------------------------------------------+


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
MT4_EXPFUNC int __stdcall gFileOpen(char *file_name,int mode)
  {
   HANDLE hFile;
   DWORD dwDesiredAccess;
	switch(mode)
   {
   case 1:
	   dwDesiredAccess=GENERIC_READ;
	   break;
   case 2:
	   dwDesiredAccess=GENERIC_WRITE;
	   break;
	case 3:
	   dwDesiredAccess=GENERIC_ALL;
	   break;

   }

 
	hFile = CreateFile(file_name,    
                   dwDesiredAccess,
                   FILE_SHARE_READ, 
                   NULL,
                   OPEN_ALWAYS,
                   FILE_ATTRIBUTE_NORMAL,
                   NULL);
	return((int)hFile);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

MT4_EXPFUNC int __stdcall gFileClose(int handle)
{
	return (CloseHandle((HANDLE)handle));

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

MT4_EXPFUNC int __stdcall gFileWrite(int handle,const char *data)
{
	DWORD dwWritten;

	int result =  WriteFile((HANDLE) handle,
						data,
						lstrlen(data),
						&dwWritten,
						NULL);

	return (result);

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

MT4_EXPFUNC char* __stdcall gFileRead(int handle,int length=0)
{
	DWORD		dwRead	= 0;
	char pBuffer[1024];
	
	int result =  ReadFile(
						(HANDLE) handle,
						pBuffer,
						length,
						&dwRead,
						NULL);

	pBuffer[strlen(pBuffer)]='\0';

	return (pBuffer);

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

MT4_EXPFUNC void __stdcall gFileSeek(int handle,int offset, int mode)
{

	SetFilePointer((HANDLE) handle,offset, NULL, mode ); 

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

MT4_EXPFUNC bool __stdcall gFileDelete(char* file_name)
{

	return (DeleteFile(file_name));
 
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+


MT4_EXPFUNC int __stdcall gFileSize(int handle)
{

	return (GetFileSize((HANDLE) handle,NULL));
  
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

void do_cross_validation()
{
	int i;
	int total_correct = 0;
	double total_error = 0;
	double sumv = 0, sumy = 0, sumvv = 0, sumyy = 0, sumvy = 0;
	double *target = Malloc(double,prob.l);

	svm_cross_validation(&prob,&param,nr_fold,target);
	if(param.svm_type == EPSILON_SVR ||
	   param.svm_type == NU_SVR)
	{
		for(i=0;i<prob.l;i++)
		{
			double y = prob.y[i];
			double v = target[i];
			total_error += (v-y)*(v-y);
			sumv += v;
			sumy += y;
			sumvv += v*v;
			sumyy += y*y;
			sumvy += v*y;
		}
		printf("Cross Validation Mean squared error = %g\n",total_error/prob.l);
		printf("Cross Validation Squared correlation coefficient = %g\n",
			((prob.l*sumvy-sumv*sumy)*(prob.l*sumvy-sumv*sumy))/
			((prob.l*sumvv-sumv*sumv)*(prob.l*sumyy-sumy*sumy))
			);
	}
	else
	{
		for(i=0;i<prob.l;i++)
			if(target[i] == prob.y[i])
				++total_correct;
		printf("Cross Validation Accuracy = %g%%\n",100.0*total_correct/prob.l);
	}
	free(target);
}


//+------------------------------------------------------------------+
//|   read in a problem (in svmlight format)                         |
//+------------------------------------------------------------------+

int read_problem(char *filename)
{
	int elements, max_index, i, j;
	FILE *fp = fopen(filename,"r");
	
	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		return(-1);
	}

	prob.l = 0;
	elements = 0;
	while(1)
	{
		int c = fgetc(fp);
		switch(c)
		{
			case '\n':
				++prob.l;
				// fall through,
				// count the '-1' element
			case ':':
				++elements;
				break;
			case EOF:
				goto out;
			default:
				;
		}
	}
out:
	rewind(fp);

	prob.y = Malloc(double,prob.l);
	prob.x = Malloc(struct svm_node *,prob.l);
	x_space = Malloc(struct svm_node,elements);

	max_index = 0;
	j=0;
	for(i=0;i<prob.l;i++)
	{
		double label;
		prob.x[i] = &x_space[j];
		fscanf(fp,"%lf",&label);
		prob.y[i] = label;

		while(1)
		{
			int c;
			do {
				c = getc(fp);
				if(c=='\n') goto out2;
			} while(isspace(c));
			ungetc(c,fp);
			if (fscanf(fp,"%d:%lf",&(x_space[j].index),&(x_space[j].value)) < 2)
			{
				fprintf(stderr,"Wrong input format at line %d\n", i+1);
				return(-1);
			}
			++j;
		}	
out2:
		if(j>=1 && x_space[j-1].index > max_index)
			max_index = x_space[j-1].index;
		x_space[j++].index = -1;
	}

	if(param.gamma == 0)
		param.gamma = 1.0/max_index;

	if(param.kernel_type == PRECOMPUTED)
		for(i=0;i<prob.l;i++)
		{
			if (prob.x[i][0].index != 0)
			{
				fprintf(stderr,"Wrong input format: first column must be 0:sample_serial_number\n");
				return(-1);
			}
			if ((int)prob.x[i][0].value <= 0 || (int)prob.x[i][0].value > max_index)
			{
				fprintf(stderr,"Wrong input format: sample_serial_number out of range\n");
				return(-1);
			}
		}

	fclose(fp);
	return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
/*
Examples of options: -s 0 -c 10 -t 1 -g 1 -r 1 -d 3 
Classify a binary data with polynomial kernel (u'v+1)^3 and C = 10 
 
options:
-s svm_type : set type of SVM (default 0)
	0 -- C-SVC
	1 -- nu-SVC
	2 -- one-class SVM
	3 -- epsilon-SVR
	4 -- nu-SVR
-t kernel_type : set type of kernel function (default 2)
	0 -- linear: u'*v
	1 -- polynomial: (gamma*u'*v + coef0)^degree
	2 -- radial basis function: exp(-gamma*|u-v|^2)
	3 -- sigmoid: tanh(gamma*u'*v + coef0)
-d degree : set degree in kernel function (default 3)
-g gamma : set gamma in kernel function (default 1/k)
-r coef0 : set coef0 in kernel function (default 0)
-c cost : set the parameter C of C-SVC, epsilon-SVR, and nu-SVR (default 1)
-n nu : set the parameter nu of nu-SVC, one-class SVM, and nu-SVR (default 0.5)
-p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)
-m cachesize : set cache memory size in MB (default 100)
-e epsilon : set tolerance of termination criterion (default 0.001)
-h shrinking: whether to use the shrinking heuristics, 0 or 1 (default 1)
-b probability_estimates: whether to train a SVC or SVR model for probability estimates, 0 or 1 (default 0)
-wi weight: set the parameter C of class i to weight*C, for C-SVC (default 1)

The k in the -g option means the number of attributes in the input data.

option -v randomly splits the data into n parts and calculates cross
validation accuracy/mean squared error on them.
*/
MT4_EXPFUNC int __stdcall SvmTrain(char *input_file_name, char *model_file_name)
{

    int cross_validation;
    // default values
	//+------------------------------------------------------------------+
	//1.)int svm_type: set type of SVM (default 0)
	//	0 -- C-SVC
	//	1 -- nu-SVC
	//	2 -- one-class SVM
	//	3 -- epsilon-SVR
	//	4 -- nu-SVR
	param.svm_type     = C_SVC;	/*C_SVC, NU_SVC, ONE_CLASS, EPSILON_SVR, NU_SVR*/
    //+------------------------------------------------------------------+
	//2.)int kernel_type: set type of kernel function (default 2)
	//	0 -- linear: u'*v
	//	1 -- polynomial	: (gamma*u'*v + coef0)^degree
	//	2 -- radial basis function: exp(-gamma*|u-v|^2)
	//	3 -- sigmoid	: tanh(gamma*u'*v + coef0)
	//	4 -- precomputed kernel (kernel values in training_set_file)
	param.kernel_type  = RBF;	/*LINEAR, POLY, RBF, SIGMOID, PRECOMPUTED */
    //+------------------------------------------------------------------+
	//3.)int degree: set degree in kernel function (default 3)
	param.degree       = 3;
	//+------------------------------------------------------------------+
	//4.)double gamma: set gamma in kernel function (default 1/k)
	param.gamma        = 0;		// 1/k
	//+------------------------------------------------------------------+
	//5.)double coef0: set coef0 in kernel function (default 0)
	param.coef0        = 0;		/*double coef0; for poly/sigmoid */
	//+------------------------------------------------------------------+	
	//6.)nu: set the parameter nu of nu-SVC, one-class SVM, and nu-SVR (default 0.5)
	param.nu           = 0.5;	/*double nu; for NU_SVC, ONE_CLASS, and NU_SVR */
	//+------------------------------------------------------------------+
	//7.)double cachesize: set cache memory size in MB (default 100)
	param.cache_size   = 100;
	//+------------------------------------------------------------------+
    //8.)cost: set the parameter C of C-SVC, epsilon-SVR, and nu-SVR (default 1)
	param.C            = 1;		/* for C_SVC, EPSILON_SVR and NU_SVR */
	//+------------------------------------------------------------------+
	//9.)double epsilon: set tolerance of termination criterion (default 0.001)
	param.eps          = 1e-3;	/* stopping criteria */
    //+------------------------------------------------------------------+
	//10.)double epsilon: set the epsilon in loss function of epsilon-SVR (default 0.1)
	param.p            = 0.1;	/*double p;	 for EPSILON_SVR */
	//+------------------------------------------------------------------+
	//11.)shrinking	: whether to use the shrinking heuristics, 0 or 1 (default 1)
	param.shrinking    = 1;		/*int shrinking;	 use the shrinking heuristics */
	//+------------------------------------------------------------------+
	//12.)probability_estimates: whether to train a SVC or SVR model for probability estimates, 0 or 1 (default 0)
	param.probability  = 0;		/*int probability;  do probability estimates */
	//+------------------------------------------------------------------+
	//13.)weight: set the parameter C of class i to weight*C, for C-SVC (default 1)
	param.nr_weight    = 0;		/*int nr_weight;		 for C_SVC */
	//14.)
	param.weight_label = NULL;	/*int *weight_label;	 for C_SVC */
	//15.)
	param.weight       = NULL;	/*double* weight;		 for C_SVC */
	//+------------------------------------------------------------------+
    //n					: n-fold cross validation mode
	cross_validation   = 0;
   
   	//char input_file_name[1024];
	//char model_file_name[1024];
	const char *error_msg;

	//parse_command_line(argc, argv, input_file_name, model_file_name);
	
	if(read_problem(input_file_name)!=0)
	{
		fprintf(stderr,"Fle not Exist:\n",input_file_name);
		return(-1);
	}
/*
	error_msg = svm_check_parameter(&prob,&param);

	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		return(-1);
	}
*/
	if(cross_validation)
	{
		do_cross_validation();
	}
	else
	{

		model = svm_train(&prob,&param);
		svm_save_model(model_file_name,model);
		svm_destroy_model(model);
	}

	svm_destroy_param(&param);
	free(prob.y);
	free(prob.x);
	free(x_space);


   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int predict(const int predict_probability,FILE *input, FILE *output)
{
	int correct = 0;
	int total = 0;
	double error = 0;
	double sumv = 0, sumy = 0, sumvv = 0, sumyy = 0, sumvy = 0;

	int svm_type=svm_get_svm_type(model);
	int nr_class=svm_get_nr_class(model);
	double *prob_estimates=NULL;
	int j;
	int retval=0;

	if(predict_probability)
	{
		if (svm_type==NU_SVR || svm_type==EPSILON_SVR)
        {
			//printf("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma=%g\n",svm_get_svr_probability(model));		
		}
		else
		{
			int *labels=(int *) malloc(nr_class*sizeof(int));
			svm_get_labels(model,labels);
			prob_estimates = (double *) malloc(nr_class*sizeof(double));
			fprintf(output,"labels");		
			for(j=0;j<nr_class;j++)
				fprintf(output," %d",labels[j]);
			fprintf(output,"\n");
			free(labels);
		}
	}
	while(1)
	{
		int i = 0;
		int c;
		double target,v;

		if (fscanf(input,"%lf",&target)==EOF)
			break;

		while(1)
		{
			if(i>=max_nr_attr-1)	// need one more for index = -1
			{
				max_nr_attr *= 2;
				x = (struct svm_node *) realloc(x,max_nr_attr*sizeof(struct svm_node));
			}

			do 
			{
				c = getc(input);
				if(c=='\n' || c==EOF) goto out2;
			} while(isspace(c));
			ungetc(c,input);
			if (fscanf(input,"%d:%lf",&x[i].index,&x[i].value) < 2)
			{
				fprintf(stderr,"Wrong input format at line %d\n", total+1);
				return(-1);
			}
			++i;
		}	

out2:
		x[i].index = -1;

		if (predict_probability && (svm_type==C_SVC || svm_type==NU_SVC))
		{
			v = svm_predict_probability(model,x,prob_estimates);
			fprintf(output,"%g ",v);
			for(j=0;j<nr_class;j++)
				fprintf(output,"%g ",prob_estimates[j]);
			fprintf(output,"\n");
		}
		else
		{
			v = svm_predict(model,x);
			fprintf(output,"%g\n",v);
		}

		if(v == target)
			++correct;
		error += (v-target)*(v-target);
		sumv += v;
		sumy += target;
		sumvv += v*v;
		sumyy += target*target;
		sumvy += v*target;
		++total;
	}
	if (svm_type==NU_SVR || svm_type==EPSILON_SVR)
	{/*
		printf("Mean squared error = %g (regression)\n",error/total);
		printf("Squared correlation coefficient = %g (regression)\n",
		       ((total*sumvy-sumv*sumy)*(total*sumvy-sumv*sumy))/
		       ((total*sumvv-sumv*sumv)*(total*sumyy-sumy*sumy))
		       );
		*/
		retval=-1;
	}
	else
	{/*
		printf("Accuracy = %g%% (%d/%d) (classification)\n",
		       (double)correct/total*100,correct,total);
	  */
	}
	if(predict_probability)
		free(prob_estimates);

	return(retval);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
MT4_EXPFUNC int __stdcall SvmPredict(									  
									 const char *test_file_name, 
									 const char *model_file_name, 
									 const char *predict_file_name
									)
{

	const int predict_probability=0;
   //probability_estimates: whether to predict_probability estimates, 
   //0 or 1 (default 0); for one-class SVM only 0 is supported
	FILE *input, *output;


	//////////////////////////////////
	input = fopen(test_file_name,"r");
	//////////////////////////////////
	if(input == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",test_file_name);
		return(-1);
	}
	//////////////////////////////////////
	output = fopen(predict_file_name,"w");
	//////////////////////////////////////
	if(output == NULL)
	{
		fprintf(stderr,"can't open output file %s\n",predict_file_name);
		return(-1);
	}
    ////////////////////////////////////////////////
	if((model=svm_load_model(model_file_name))==0)
    ////////////////////////////////////////////////
	{
		fprintf(stderr,"can't open model file %s\n",model_file_name);
		return(-1);
	}
	
	line = (char *) malloc(max_line_len*sizeof(char));
	x = (struct svm_node *) malloc(max_nr_attr*sizeof(struct svm_node));

	//if(predict_probability)
	//{
	//	if(svm_check_probability_model(model)==0)
	//	{
	//		fprintf(stderr,"Model does not support probabiliy estimates\n");
	//		return(-1);
	//	}
	//}
	//else
	//{
	//	if(svm_check_probability_model(model)!=0)
	//	{
	//		//printf("Model supports probability estimates, but disabled in prediction.\n");
	//	}
	//}
	
	if(predict(predict_probability,input,output)!=0)
    {
			fprintf(stderr,"Predict error.\n");
			return(-1);
    }

	svm_destroy_model(model);

	free(line);
	free(x);
	fclose(input);
	fclose(output);


  return(0);
}


char* readline(FILE *input)
{
	int len;
	
	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line, max_line_len);
		len = strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void output_target(double value,FILE *fps)
{
	if(y_scaling)
	{
		if(value == y_min)
			value = y_lower;
		else if(value == y_max)
			value = y_upper;
		else value = y_lower + (y_upper-y_lower) *
			     (value - y_min)/(y_max-y_min);
	}
	fprintf(fps,"%g ",value);
}

void output(int index, double value,FILE *fps)
{
	// skip single-valued attribute 
	if(feature_max[index] == feature_min[index])
		return;

	if(value == feature_min[index])
		value = lower;
	else if(value == feature_max[index])
		value = upper;
	else
		value = lower + (upper-lower) * 
			(value-feature_min[index])/
			(feature_max[index]-feature_min[index]);

	if(value != 0)
		fprintf(fps,"%d:%g ",index, value);
}


MT4_EXPFUNC int __stdcall SvmScale(double l,
								   double u,
								   char *input_file_name, 
								   /*char *save_filename,*/
								   char *scale_file_name
								   )
{
	int i,index;
	FILE *fp,*fps;
/*
Scaled sets with default parameters
$./svm-scale -s range1 train.1 > train.1.scale
$./svm-scale -r range1 test.1 > test.1.scale
$./svm-train train.1.scale
$./svm-predict test.1.scale train.1.scale.model test.1.predict	
...............................................................
*/


	//char *save_filename = NULL;
	//char *restore_filename = NULL;

/*
			case 'l': lower = atof(argv[i]); break;
			case 'u': upper = atof(argv[i]); break;
			case 'y':
				y_lower = atof(argv[i]);
				++i;
				y_upper = atof(argv[i]);
				y_scaling = 1;
				break;
			case 's': save_filename = argv[i]; break;
			case 'r': restore_filename = argv[i]; break;
			default:
				fprintf(stderr,"unknown option\n");
				exit(1);
*/
	lower=l;
	upper=u;
	y_lower = -1.0;
	y_upper = 1.0;
	y_scaling = 0;

	fp=fopen(input_file_name,"r");
	fps=fopen(scale_file_name,"w");
	
	if(fp==NULL)
	{
		fprintf(stderr,"can't open file %s\n", input_file_name);
		return(-1);
	}
	if(fps==NULL)
	{
		fprintf(stderr,"can't open file %s\n", scale_file_name);
		return(-1);
	}

	line = (char *) malloc(max_line_len*sizeof(char));


#define SKIP_TARGET while(isspace(*p)) ++p; while(!isspace(*p)) ++p;

#define SKIP_ELEMENT\
	while(*p!=':') ++p;\
	++p;\
	while(isspace(*p)) ++p;\
	while(*p && !isspace(*p)) ++p;
	
	// assumption: min index of attributes is 1 
	// pass 1: find out max index of attributes 
	max_index = 0;
	while(readline(fp)!=NULL)
	{
		char *p=line;

		SKIP_TARGET

		while(sscanf(p,"%d:%*f",&index)==1)
		{
			max_index = max(max_index, index);
			SKIP_ELEMENT
		}		
	}
	
	feature_max = (double *)malloc((max_index+1)* sizeof(double));
	feature_min = (double *)malloc((max_index+1)* sizeof(double));
	
	if(feature_max == NULL || feature_min == NULL)
	{
		fprintf(stderr,"can't allocate enough memory\n");
		return(-1);
	}

	for(i=0;i<=max_index;i++)
	{
		feature_max[i]=-DBL_MAX;
		feature_min[i]=DBL_MAX;
	}

	rewind(fp);

	// pass 2: find out min/max value 
	while(readline(fp)!=NULL)
	{
		char *p=line;
		int next_index=1;
		double target;
		double value;

		sscanf(p,"%lf",&target);
		y_max = max(y_max,target);
		y_min = min(y_min,target);
		
		SKIP_TARGET

		while(sscanf(p,"%d:%lf",&index,&value)==2)
		{
			for(i=next_index;i<index;i++)
			{
				feature_max[i]=max(feature_max[i],0);
				feature_min[i]=min(feature_min[i],0);
			}
			
			feature_max[index]=max(feature_max[index],value);
			feature_min[index]=min(feature_min[index],value);

			SKIP_ELEMENT
			next_index=index+1;
		}		

		for(i=next_index;i<=max_index;i++)
		{
			feature_max[i]=max(feature_max[i],0);
			feature_min[i]=min(feature_min[i],0);
		}	
	}

	rewind(fp);


	// pass 2.5: save/restore feature_min/feature_max 
/*	
//	if(restore_filename)
//	{
		FILE *fp_restore = fopen(restore_filename,"r");
		int idx, c;
		double fmin, fmax;
		
		if(fp_restore==NULL)
		{
			fprintf(stderr,"can't open file %s\n", restore_filename);
			exit(1);
		}
		if((c = fgetc(fp_restore)) == 'y')
		{
			fscanf(fp_restore, "%lf %lf\n", &y_lower, &y_upper);
			fscanf(fp_restore, "%lf %lf\n", &y_min, &y_max);
			y_scaling = 1;
		}
		else
			ungetc(c, fp_restore);

		if (fgetc(fp_restore) == 'x') {
			fscanf(fp_restore, "%lf %lf\n", &lower, &upper);
			while(fscanf(fp_restore,"%d %lf %lf\n",&idx,&fmin,&fmax)==3)
			{
				if(idx<=max_index)
				{
					feature_min[idx] = fmin;
					feature_max[idx] = fmax;
				}
			}
		}
		fclose(fp_restore);
//	}
*/	
/*
//	if(save_filename)
//	{
		FILE *fp_save = fopen(save_filename,"w");
		if(fp_save==NULL)
		{
			fprintf(stderr,"can't open file %s\n", save_filename);
			return(-1);
		}
		if(y_scaling)
		{
			fprintf(fp_save, "y\n");
			fprintf(fp_save, "%.16g %.16g\n", y_lower, y_upper);
			fprintf(fp_save, "%.16g %.16g\n", y_min, y_max);
		}
		fprintf(fp_save, "x\n");
		fprintf(fp_save, "%.16g %.16g\n", lower, upper);
		for(i=1;i<=max_index;i++)
		{
			if(feature_min[i]!=feature_max[i])
				fprintf(fp_save,"%d %.16g %.16g\n",i,feature_min[i],feature_max[i]);
		}
		fclose(fp_save);
//	}
*/	
	// pass 3: scale 

	while(readline(fp)!=NULL)
	{
		char *p=line;
		int next_index=1;
		int index;
		double target;
		double value;
		
		sscanf(p,"%lf",&target);
		output_target(target,fps);

		SKIP_TARGET

		while(sscanf(p,"%d:%lf",&index,&value)==2)
		{
			for(i=next_index;i<index;i++)
				output(i,0,fps);
			
			output(index,value,fps);

			SKIP_ELEMENT
			next_index=index+1;
		}		

		for(i=next_index;i<=max_index;i++)
			output(i,0,fps);

		fprintf(fps,"\n");
	}

	free(line);
	fclose(fp);
	fclose(fps);


  return(0);
}